/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _ALSA_H_
#define _ALSA_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_LIBASOUND
extern "C" {
#include <sys/asoundlib.h>
}
#include "audio.h"

namespace Tuner {
	class Alsa: public Audio {
		public:
			Alsa() throw(Error::AudioError);
		       ~Alsa();

		bool	openDevice(AudioInfo *, bool, bool)
				throw(Error::AudioError);
		bool	closeDevice() throw();

		bool	play(unsigned char *, int) throw(Error::AudioError);
		bool	record(unsigned char *, int) throw(Error::AudioError);

		bool	avail(int *, int *) throw(Error::AudioError);
		bool	free(int *, int *) throw(Error::AudioError);

		bool	isOpen() throw();

		bool	sync() throw(Error::AudioError);
		bool	stop() throw(Error::AudioError);

		const AudioInfo *getInfo() throw();

		private:
			Alsa(const Alsa &);
		const Alsa &operator=(const Alsa &);

		bool	setParams(AudioInfo *);
		bool	getParams(AudioInfo *);

		bool convParams(snd_pcm_channel_setup_t *, AudioInfo *);
		const snd_pcm_channel_params_t convInfo(AudioInfo *);

		void	throwError(std::string, Error::AudioErrorID,
				std::string) throw(Error::AudioError);

		snd_pcm_t *handle;
		bool open;

		AudioInfo info;
		bool setBuf;
		int lastError;
	};

	inline const AudioInfo *Alsa::getInfo() throw() { return &info; }
}
#endif

#endif

