/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FFTCALC_H_
#define _FFTCALC_H_

#include <new>
#include <vector>
#include "audio.h"
#include "calc.h"
#include "fft.h"
#include "gate.h"
#include "window.h"

namespace Tuner {
	class FftCalc: public Calc {
		public:
			FftCalc(int, SamplingRate, Format, Window::WindowType,
				FftCalcMode = PEAK_SEARCH, bool = false, int = 0, int = 0,
				int = 0, int = 0) throw(std::bad_alloc);
		   ~FftCalc();

		void setAudioParams(SamplingRate, Format, bool = false);
		double operator()(unsigned char *) throw();

		private:
			FftCalc(const FftCalc &);
		const FftCalc &operator=(const FftCalc &);

		struct FftPeak {
				FftPeak() throw();
			void reset() throw();

			int index;
			double value;
			bool isMaxHarm;
			int harmCount;
		};
		class CmpVal;
		class CmpIndex;

		friend class CmpVal;
		friend class CmpIndex;

		class CmpVal {
			public:
			bool operator()(const FftPeak &, const FftPeak &) throw();
		};

		class CmpIndex {
			public:
			bool operator()(const FftPeak &, const FftPeak &) throw();
		};

		static const int PeakNumber = 20;

		static bool peakInRange(int, int, double *) throw();
		static bool isPeak(int, double *) throw();

		void fillPeakBuf(FftPeak &) throw();
		void resetPeaks() throw();
		void addPeak(int, double) throw();

		void doFft(unsigned char *) throw();
		double countMethod() throw();
		double maxMethod() throw();

		FftCalcMode mode;
		double (FftCalc:: *modeFun)();

		int points;
		SamplingRate rate;
		Format fmt;

		Window::WindowType wt;
		bool useGate;
		int percent;
		int hold;
		int rel;
		int att;

		Fft<int> *pInt;
		Fft<unsigned char> *pChar;
		Gate<int> *iGate;
		Gate<unsigned char> *cGate;

		int *iBuf;
		double *fftOut;
		int outSize;

		vector<FftPeak> peakBuf;
	};

	inline FftCalc::FftPeak::FftPeak() throw() { reset(); }
	inline void FftCalc::FftPeak::reset() throw()
	{ index = 0, value = 0, isMaxHarm = false, harmCount = 0; }

	inline bool FftCalc::CmpVal::operator()(const FftPeak &p1, const FftPeak &p2) throw()
	{ return p1.value < p2.value; }

	inline bool FftCalc::CmpIndex::operator()(const FftPeak &p1, const FftPeak &p2) throw()
	{ return p1.index < p2.index; }
}

#endif

