/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GATE_H_
#define _GATE_H_

namespace Tuner {
	template<typename T>
	class Gate {
		public:
			Gate(int, T, T, int = 0, int = 0, int = 0) throw();
		   ~Gate();

		bool	apply(T *, int) throw();

		private:
			Gate(const Gate &);
		const Gate &operator=(const Gate &);

		int rate, holdTicks, releaseTicks, attackTicks;
		double cRel, cAtt;
		T center, level;
	};

	template <typename T>
	Gate<T>::Gate(int r, T c, T l, int ht, int rt, int at) throw():
		rate(r), center(c), level(l)
	{
		holdTicks = ht * rate / 1000;
		releaseTicks = rt * rate / 1000;
		attackTicks = at * rate / 1000;
		cRel = releaseTicks ? 1.0 / releaseTicks: 1.0;
		cAtt = attackTicks ? 1.0 / attackTicks: 1.0;
	}

	template <typename T>
	Gate<T>::~Gate() { }

	template <typename T>
	bool
	Gate<T>::apply(T *data, int len) throw()
	{
		int i, hold = 0;
		bool isOn = false, dec = false;
		double fact = 1.0;

		if(level == center)
			return false;

		for(i = 0; i < len; i++) {
			if(data[i] >= (center - level) && data[i] <= (center - 1 + level)) {
				if(!isOn) {
					hold++;
					if(hold > holdTicks)
						isOn = dec = true;
				}
				if(isOn) {
					if(dec) {
						fact -= cRel;
						if(fact < 0.0) {
							fact = 0;
							dec = false;
							data[i] = center;
						} else
							data[i] = static_cast<T>(center + (data[i] - center) * fact);
					} else
						data[i] = center;
				}
			} else {
				if(isOn) {
					fact += cAtt;
					if(fact > 1.0) {
						fact = 1.0;
						isOn = false;
					} else
						data[i] = static_cast<T>(center + (data[i] - center) * fact);
				}
				hold = 0;
			}
		}

		return true;
	}
}

#endif

