/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GENCONF_H_
#define _GENCONF_H_

#include <string>
#include "conferror.h"
#include "conf.h"

namespace Config {
	using Tuner::AudioDriver;
	using Tuner::DetectionMode;
	using Tuner::AudioInfo;
	using Tuner::CalcInfo;
	using Tuner::NoteInfo;

	class GenConf {
		public:
			GenConf(int &, char **&, AudioDriver = Tuner::OSS_DRIVER,
				AudioDriver = Tuner::OSS_DRIVER, DetectionMode = Tuner::FFTCALC,
				AudioInfo * = 0, AudioInfo * = 0, CalcInfo * = 0,
				NoteInfo * = 0) throw(Error::ConfError);
		   ~GenConf();

		bool setInfo(AudioDriver, AudioDriver, DetectionMode, const AudioInfo *,
			const AudioInfo *, const CalcInfo *, const NoteInfo *) throw();

		AudioDriver &getAudioDriver() throw();
		AudioDriver &getAudioDriverPlay() throw();
		DetectionMode &getDetectionMode() throw();

		AudioInfo &getAudioInfo() throw();
		AudioInfo &getAudioInfoPlay() throw();
		CalcInfo &getCalcInfo() throw();
		NoteInfo &getNoteInfo() throw();

		private:
			GenConf(const GenConf &);
		const GenConf &operator=(const GenConf &);

		bool load(Config::ConfFile &) throw(Error::ConfError, Error::AudioError);
		bool loadDefaults() throw();
		void save() throw();
		bool save(Config::ConfFile &) throw(Error::ConfError, Error::AudioError);

		void update() throw();
		bool sync() throw();

		void parseCmdLine(int &, char **&) throw(Error::ConfError);
		void parseEnv() throw(Error::ConfError);

		AudioInfo audio, audioPlay;
		CalcInfo fft;
		NoteInfo note;

		AudioDriver audr, audrPlay;
		DetectionMode dm;

		std::string inConf, outConf;
	};

	inline AudioInfo &
	GenConf::getAudioInfo() throw() { return audio; }
	inline AudioInfo &
	GenConf::getAudioInfoPlay() throw() { return audioPlay; }
	inline CalcInfo &
	GenConf::getCalcInfo() throw() { return fft; }
	inline NoteInfo &
	GenConf::getNoteInfo() throw() { return note; }
	inline AudioDriver &
	GenConf::getAudioDriver() throw() { return audr; }
	inline AudioDriver &
	GenConf::getAudioDriverPlay() throw() { return audrPlay; }
	inline DetectionMode &
	GenConf::getDetectionMode() throw() { return dm; }
}

#endif

