/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <vector>
#include <gnome.h>
#include <unistd.h>
#include "gnomebase.h"
#include "guituner.h"
#include "playerthread.h"
#include "support.h"

namespace Interface {
	void addNotebook(GtkWidget *, GtkWidget *) throw();
	void addFooter(GtkWidget *, GtkWidget *) throw();
	void addTunerConf(GtkWidget *, GtkWidget *) throw();
	void addAudioConf(GtkWidget *, GtkWidget *) throw();
	void addOtherConf(GtkWidget *, GtkWidget *) throw();

	void collectTunerInfo(GtkWidget *, Tuner::DetectionMode &, Tuner::CalcInfo &);
	void collectAudioInfo(GtkWidget *, Tuner::AudioDriver &ad,
		Tuner::AudioDriver &adp, Tuner::AudioInfo &);
	void collectOtherInfo(GtkWidget *, Tuner::NoteInfo &);

	int onDefClicked(GtkWidget *, gpointer);
	int onPrefClicked(GtkWidget *, gpointer);
	int onPrefCancClicked(GtkWidget *, gpointer);
	int onPresetSelected(GtkWidget *, gint, gint,GdkEventButton *, gpointer);

	int onCloseClicked(GtkWidget *, gpointer);
	int onPlayClicked1(GtkWidget *, gpointer);
	int onPlayClicked2(GtkWidget *, gpointer);
	int onPlayClicked3(GtkWidget *, gpointer);
	int onPlayClicked4(GtkWidget *, gpointer);
	int onPlayClicked5(GtkWidget *, gpointer);
	int onPlayClicked6(GtkWidget *, gpointer);

	void loadBtnLabels(GtkWidget *, Player::PlayerThread *, int = 0) throw();

	struct PInfo {
		Player::PlayerThread *pt;
		GtkWidget *win;
	};

	int (*handle[6])(GtkWidget *, gpointer) = {
		onPlayClicked1,
		onPlayClicked2,
		onPlayClicked3,
		onPlayClicked4,
		onPlayClicked5,
		onPlayClicked6
	};

	struct OptMenuItem {
		GtkWidget *wid;
		char *content;
	};

	OptMenuItem ptsOpt[3] = {
		{ 0, "65536" },
		{ 0, "32768" },
		{ 0, "16384" }
	};

	OptMenuItem winOpt[7] = {
		{ 0, "None" },
		{ 0, "Rectangular" },
		{ 0, "Hanning" },
		{ 0, "Parzen" },
		{ 0, "Bartlett" },
		{ 0, "Hamming" },
		{ 0, "Blackman" }
	};

	OptMenuItem srsOpt[5] = {
		{ 0, "44100" },
		{ 0, "32000" },
		{ 0, "22050" },
		{ 0, "16000" },
		{ 0, "11025" }
	};

	OptMenuItem fmtOpt[2] = {
		{ 0, "8 bits" },
		{ 0, "16 bits" }
	};

	OptMenuItem ncOpt[2] = {
		{ 0, "English" },
		{ 0, "Italian" }
	};
}

void
Interface::GnomeBase::openMessageBox(const std::string &t, DialogType d) throw()
{
	GtkWidget *errorBox;
	GtkWidget *dialogVbox;
	GtkWidget *button;
	GtkWidget *dialogActionArea;

	errorBox = gnome_message_box_new(t.c_str(),
		d == ERROR ? GNOME_MESSAGE_BOX_ERROR: GNOME_MESSAGE_BOX_ERROR, 0);
	gtk_object_set_data(GTK_OBJECT(errorBox), "ErrorBox", errorBox);
	gtk_window_set_title(GTK_WINDOW(errorBox), d == ERROR ? _("Error"): _("Info"));
	gtk_window_set_policy(GTK_WINDOW(errorBox), FALSE, FALSE, FALSE);

	dialogVbox = GNOME_DIALOG(errorBox)->vbox;
	gtk_object_set_data(GTK_OBJECT(errorBox), "dialogVbox", dialogVbox);
	gtk_widget_show(dialogVbox);

	gnome_dialog_append_button(GNOME_DIALOG(errorBox), GNOME_STOCK_BUTTON_OK);
	button = static_cast<GtkWidget *>(g_list_last(GNOME_DIALOG(errorBox)->buttons)->data);
	gtk_widget_ref(button);
	gtk_object_set_data_full(GTK_OBJECT(errorBox), "button", button,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

	dialogActionArea = GNOME_DIALOG(errorBox)->action_area;
	gtk_widget_ref(dialogActionArea);
	gtk_object_set_data_full(GTK_OBJECT(errorBox), "dialogActionArea",
		dialogActionArea, (GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_show(errorBox);
}

void
Interface::GnomeBase::openPlayWindow(Config::GenConf &gc) throw()
{
	GtkWidget *window;
	GtkWidget *fixed;
	GtkWidget *scrWin;
	GtkWidget *presetList;
	GtkWidget *btn[6];
	GtkWidget *closeBtn;
	GtkWidget *play;
	GtkWidget *hseparator;
	GtkWidget *lab[6];
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data(GTK_OBJECT(window), "window", window);
	gtk_window_set_title(GTK_WINDOW(window), _("Play Window"));
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
	gtk_widget_set_usize(window, 264, 336);

	fixed = gtk_fixed_new();
	gtk_widget_ref(fixed);
	gtk_object_set_data_full(GTK_OBJECT(window), "fixed", fixed,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fixed);
	gtk_container_add(GTK_CONTAINER(window), fixed);

	scrWin = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrWin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_fixed_put(GTK_FIXED(fixed), scrWin, 8, 152);
	gtk_widget_set_uposition(scrWin, 8, 152);
	gtk_widget_show(scrWin);

	char *title = _("Presets");
	presetList = gtk_clist_new_with_titles(1, &title);
	gtk_clist_set_selection_mode(GTK_CLIST(presetList), GTK_SELECTION_SINGLE);
	gtk_widget_ref(presetList);
	gtk_object_set_data_full(GTK_OBJECT(window), "presetList", presetList,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(presetList);
	gtk_clist_column_title_passive(GTK_CLIST(presetList), 0);
	gtk_widget_set_usize(scrWin, 248, 125);
	gtk_container_add(GTK_CONTAINER(scrWin), presetList);

	PInfo *pi;
	try {
		pi = new PInfo;
	 } catch(std::bad_alloc e) {
		gtk_widget_destroy(window);
		openMessageBox(e.what(), ERROR);
		return;
	}

	Player::PlayerThread *pt;

	try {
		pt = new Player::PlayerThread(gc);
	 } catch(std::bad_alloc e) {
		gtk_widget_destroy(window);
		openMessageBox(e.what(), ERROR);
		return;
	}

	try {
		pt->start();
		pi->pt = pt;
		pi->win = window;
	} catch(std::exception &e) {
		gtk_widget_destroy(window);
		delete pt;
		delete pi;
		openMessageBox(e.what(), ERROR);
		return;
	}

	for(int i = 0; i < 6; ++i) {
		char buf[20];
		std::sprintf(buf, "btn[%d]", i);
		btn[i] = gtk_button_new();
		gtk_widget_ref(btn[i]);
		gtk_object_set_data_full(GTK_OBJECT(window), buf, btn[i],
			(GtkDestroyNotify)gtk_widget_unref);
		gtk_widget_show(btn[i]);
		gtk_fixed_put(GTK_FIXED(fixed), btn[i], 16 + 40 * i, 32);
		gtk_widget_set_uposition(btn[i], 16 + 40 * i, 32);
		gtk_widget_set_usize(btn[i], 32, 32);
		GtkWidget *label = gtk_label_new("");
		gtk_widget_ref(label);
		std::sprintf(buf, "btnLabel[%d]", i);
		gtk_object_set_data_full(GTK_OBJECT(window), buf, label,
			(GtkDestroyNotify)gtk_widget_unref);
		gtk_container_add(GTK_CONTAINER(btn[i]), label);
		gtk_widget_show(label);
		gtk_signal_connect(GTK_OBJECT(btn[i]), "clicked",
			GTK_SIGNAL_FUNC(handle[i]), pi);
	}

	closeBtn = gnome_stock_button(GNOME_STOCK_BUTTON_CLOSE);
	gtk_widget_ref(closeBtn);
	gtk_object_set_data_full(GTK_OBJECT(window), "closeBtn", closeBtn,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(closeBtn);
	gtk_fixed_put(GTK_FIXED(fixed), closeBtn, 178, 288);
	gtk_widget_set_uposition(closeBtn, 170, 288);
	gtk_widget_set_usize(closeBtn, 78, 32);
	gtk_container_set_border_width(GTK_CONTAINER(closeBtn), 2);
	gtk_tooltips_set_tip(tooltips, closeBtn, _("Close this window"), NULL);

	play = gtk_label_new(_("Play"));
	gtk_widget_ref(play);
	gtk_object_set_data_full(GTK_OBJECT(window), "play", play,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(play);
	gtk_fixed_put(GTK_FIXED(fixed), play, 8, 8);
	gtk_widget_set_uposition(play, 8, 8);
	gtk_widget_set_usize(play, 42, 18);

	hseparator = gtk_hseparator_new();
	gtk_widget_ref(hseparator);
	gtk_object_set_data_full(GTK_OBJECT(window), "hseparator", hseparator,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hseparator);
	gtk_fixed_put(GTK_FIXED(fixed), hseparator, 8, 120);
	gtk_widget_set_uposition(hseparator, 8, 120);
	gtk_widget_set_usize(hseparator, 248, 16);

	const char *labs[6] = { "6th", "5th", "4th", "3rd", "2nd", "1st" };
	for(int i = 0; i < 6; ++i) {
		lab[i] = gtk_label_new(_(labs[i]));
		gtk_widget_ref(lab[i]);
		gtk_object_set_data_full(GTK_OBJECT(window), "lab", lab[i],
			(GtkDestroyNotify)gtk_widget_unref);
		gtk_widget_show(lab[i]);
		gtk_fixed_put(GTK_FIXED(fixed), lab[i], 16 + 40 * i, 72);
		gtk_widget_set_uposition(lab[i], 16 + 40 * i, 72);
		gtk_widget_set_usize(lab[i], 32, 18);
	}

	gtk_object_set_data(GTK_OBJECT(window), "tooltips", tooltips);

	loadBtnLabels(window, pt);

	std::vector<Player::Preset> v = pt->listPresets();
	char **ptr = new char*;
	for(int i = 0; i < v.size(); ++i) {
		*ptr = const_cast<char *>(v[i].readName().c_str());
		gtk_clist_append(GTK_CLIST(presetList), ptr);
	}
	delete ptr;
	gtk_clist_select_row(GTK_CLIST(presetList), 0,0);

	gtk_signal_connect(GTK_OBJECT(closeBtn), "clicked",
		GTK_SIGNAL_FUNC(onCloseClicked), pi);
	gtk_signal_connect(GTK_OBJECT(presetList), "select_row",
		GTK_SIGNAL_FUNC(onPresetSelected), pi);

	gtk_widget_show(window);
}

void
Interface::loadBtnLabels(GtkWidget *w, Player::PlayerThread *pt, int s) throw()
{
	Player::Preset p = pt->select(s);
	for(int i = 0; i < 6; ++i) {
		char buf[10];
		std::sprintf(buf, "btnLabel[%d]", i);
		if(GtkWidget *l = lookupWidget(w, buf)) {
			gtk_label_set_text(GTK_LABEL(l), p[i].readName().c_str());
			gtk_widget_show(l);
		}
	}
}

int
Interface::onCloseClicked(GtkWidget *w, gpointer p)
{
	static_cast<PInfo *>(p)->pt->stop();
	delete static_cast<PInfo *>(p)->pt;
	gtk_widget_destroy(static_cast<PInfo *>(p)->win);
	delete static_cast<PInfo *>(p);
	return TRUE;
}

int
Interface::onPlayClicked1(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(0);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}

int
Interface::onPlayClicked2(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(1);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}


int
Interface::onPlayClicked3(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(2);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}

int
Interface::onPlayClicked4(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(3);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}

int
Interface::onPlayClicked5(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(4);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}

int
Interface::onPlayClicked6(GtkWidget *w, gpointer p)
{
	try {
		static_cast<PInfo *>(p)->pt->play(5);
		return TRUE;
	} catch(std::exception &e) {
		GnomeBase::openMessageBox(e.what(), GnomeBase::ERROR);
	}
	return FALSE;
}

int
Interface::onPresetSelected(GtkWidget *l, gint row, gint col,
	GdkEventButton *e, gpointer p)
{
	loadBtnLabels(static_cast<PInfo *>(p)->win,
		static_cast<PInfo *>(p)->pt, row);
	gtk_widget_show(l);

	return TRUE;
}


void
Interface::GnomeBase::openPrefWindow() throw()
{
	GtkWidget *prefWindow;
	GtkWidget *vBox;

	prefWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data(GTK_OBJECT(prefWindow), "prefWindow", prefWindow);
	gtk_window_set_title(GTK_WINDOW(prefWindow), _("Preferences"));
	gtk_window_set_policy(GTK_WINDOW(prefWindow), FALSE, FALSE, TRUE);

	vBox = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vBox);
	gtk_object_set_data_full(GTK_OBJECT(prefWindow), "vBox", vBox,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vBox);
	gtk_container_add(GTK_CONTAINER(prefWindow), vBox);

	addNotebook(prefWindow, vBox);
	addFooter(prefWindow, vBox);

	gtk_widget_show(prefWindow);
}

void
Interface::addNotebook(GtkWidget *win, GtkWidget *box) throw()
{
	GtkWidget *confNotebook = gtk_notebook_new();
	gtk_widget_ref(confNotebook);
	gtk_object_set_data_full(GTK_OBJECT(win), "confNotebook", confNotebook,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(confNotebook);
	gtk_box_pack_start(GTK_BOX(box), confNotebook, TRUE, TRUE, 0);

	addTunerConf(win, confNotebook);
	addAudioConf(win, confNotebook);
	addOtherConf(win, confNotebook);
}

void
Interface::addFooter(GtkWidget *win, GtkWidget *box) throw()
{
	GtkWidget *footer = gtk_fixed_new();
	gtk_widget_ref(footer);
	gtk_object_set_data_full(GTK_OBJECT(win), "footer", footer,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(footer);
	gtk_box_pack_start(GTK_BOX(box), footer, FALSE, TRUE, 0);
	gtk_widget_set_usize(footer, -2, 46);

	GtkWidget *prefCancel = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
	gtk_widget_ref(prefCancel);
	gtk_object_set_data_full(GTK_OBJECT(win), "prefCancel", prefCancel,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(prefCancel);
	gtk_fixed_put(GTK_FIXED(footer), prefCancel, 360, 8);
	gtk_widget_set_uposition(prefCancel, 360, 8);
	gtk_widget_set_usize (prefCancel, 100, 32);

	GtkWidget *prefOk = gnome_stock_button(GNOME_STOCK_BUTTON_OK);
	gtk_widget_ref(prefOk);
	gtk_object_set_data_full(GTK_OBJECT(win), "prefOk", prefOk,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(prefOk);
	gtk_fixed_put(GTK_FIXED(footer), prefOk, 248, 8);
	gtk_widget_set_uposition(prefOk, 248, 8);
	gtk_widget_set_usize(prefOk, 100, 32);

	gtk_signal_connect(GTK_OBJECT(prefCancel), "clicked",
		GTK_SIGNAL_FUNC(onPrefCancClicked), win);
	gtk_signal_connect(GTK_OBJECT(prefOk), "clicked",
		GTK_SIGNAL_FUNC (onPrefClicked), win);
}

void
Interface::addTunerConf(GtkWidget *win, GtkWidget *nb) throw()
{
	Tuner::CalcInfo ci = GuiTuner::getConf()->getCalcInfo();
	GSList *methodGroup = NULL;

	GtkWidget *fixed = gtk_fixed_new();
	gtk_widget_ref(fixed);
	gtk_object_set_data_full(GTK_OBJECT(win), "fixed2", fixed,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fixed);
	gtk_container_add(GTK_CONTAINER(nb), fixed);
	gtk_widget_set_usize(fixed, 480, 160);

	GtkWidget *metLab = gtk_label_new(_("Method:"));
	gtk_widget_ref(metLab);
	gtk_object_set_data_full(GTK_OBJECT(win), "metLab", metLab,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(metLab);
	gtk_fixed_put(GTK_FIXED(fixed), metLab, 8, 8);
	gtk_widget_set_uposition(metLab, 8, 8);
	gtk_widget_set_usize(metLab, 70, 25);
	gtk_misc_set_padding(GTK_MISC(metLab), 5, 0);

	GtkWidget *harmButt = gtk_radio_button_new_with_label(methodGroup,
		_("Harmonics Count"));
	methodGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(harmButt));
	gtk_widget_ref(harmButt);
	gtk_object_set_data_full(GTK_OBJECT(win), "harmButt", harmButt,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(harmButt);
	gtk_fixed_put(GTK_FIXED(fixed), harmButt, 24, 32);
	gtk_widget_set_uposition(harmButt, 24, 32);
	gtk_widget_set_usize(harmButt, 155, 28);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(harmButt),
		ci.mode == Tuner::PEAK_SEARCH);

	GtkWidget *maxButt = gtk_radio_button_new_with_label(methodGroup, _("Max Peak "));
	methodGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(maxButt));
	gtk_widget_ref(maxButt);
	gtk_object_set_data_full(GTK_OBJECT(win), "maxButt", maxButt,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(maxButt);
	gtk_fixed_put(GTK_FIXED(fixed), maxButt, 24, 56);
	gtk_widget_set_uposition(maxButt, 24, 56);
	gtk_widget_set_usize(maxButt, 155, 28);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(maxButt),
		ci.mode == Tuner::MAX_SEARCH);

	GtkWidget *ptsMenu = gtk_option_menu_new();
	gtk_widget_ref(ptsMenu);
	gtk_object_set_data_full(GTK_OBJECT (win), "ptsMenu", ptsMenu,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(ptsMenu);
	gtk_fixed_put(GTK_FIXED(fixed), ptsMenu, 112, 88);
	gtk_widget_set_uposition(ptsMenu, 112, 88);
	gtk_widget_set_usize(ptsMenu, 130, 30);
	GtkWidget *ptsMenu_menu = gtk_menu_new();

	int val = 0;
	for(int i = 0; i < 3; ++i) {
		ptsOpt[i].wid = gtk_menu_item_new_with_label(_(ptsOpt[i].content));
		gtk_widget_show(ptsOpt[i].wid);
		gtk_menu_append(GTK_MENU(ptsMenu_menu), ptsOpt[i].wid);

		if(atoi(ptsOpt[i].content) == ci.points)
			val = i;
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(ptsMenu), ptsMenu_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(ptsMenu), val);

	GtkWidget *winMenu = gtk_option_menu_new();
	gtk_widget_ref(winMenu);
	gtk_object_set_data_full(GTK_OBJECT(win), "winMenu", winMenu,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(winMenu);
	gtk_fixed_put(GTK_FIXED(fixed), winMenu, 112, 120);
	gtk_widget_set_uposition(winMenu, 112, 120);
	gtk_widget_set_usize(winMenu, 130, 30);
	GtkWidget *winMenu_menu = gtk_menu_new();

	for(int i = 0; i < 7; ++i) {
		winOpt[i].wid = gtk_menu_item_new_with_label(_(winOpt[i].content));
		gtk_widget_show(winOpt[i].wid);
		gtk_menu_append(GTK_MENU(winMenu_menu), winOpt[i].wid);
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(winMenu), winMenu_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(winMenu), ci.wtype);

	GtkWidget *winLab = gtk_label_new(_("Fft Window:"));
	gtk_widget_ref(winLab);
	gtk_object_set_data_full(GTK_OBJECT(win), "winLab", winLab,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(winLab);
	gtk_fixed_put(GTK_FIXED(fixed), winLab, 8, 128);
	gtk_widget_set_uposition(winLab, 8, 128);
	gtk_widget_set_usize(winLab, 96, 20);

	GtkWidget *pts = gtk_label_new(_("Fft Points:"));
	gtk_widget_ref(pts);
	gtk_object_set_data_full(GTK_OBJECT (win), "pts", pts,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(pts);
	gtk_fixed_put(GTK_FIXED(fixed), pts, 8, 96);
	gtk_widget_set_uposition(pts, 8, 96);
	gtk_widget_set_usize(pts, 96, 20);

	GtkWidget *vseparator = gtk_vseparator_new();
	gtk_widget_ref(vseparator);
	gtk_object_set_data_full(GTK_OBJECT(win), "vseparator", vseparator,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vseparator);
	gtk_fixed_put(GTK_FIXED(fixed), vseparator, 240, 0);
	gtk_widget_set_uposition(vseparator, 240, 0);
	gtk_widget_set_usize (vseparator, 16, 160);

	GtkWidget *gOn = gtk_label_new(_("Gate On"));
	gtk_widget_ref(gOn);
	gtk_object_set_data_full(GTK_OBJECT(win), "gOn", gOn,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(gOn);
	gtk_fixed_put(GTK_FIXED(fixed), gOn, 256, 48);
	gtk_widget_set_uposition(gOn, 256, 48);
	gtk_widget_set_usize(gOn, 70, 20);

	GtkWidget *gateOn = gtk_check_button_new_with_label(_("Use Gate"));
	gtk_widget_ref(gateOn);
	gtk_object_set_data_full(GTK_OBJECT(win), "gateOn", gateOn,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(gateOn);
	gtk_fixed_put(GTK_FIXED(fixed), gateOn, 256, 8);
	gtk_widget_set_uposition(gateOn, 256, 8);
	gtk_widget_set_usize(gateOn, 121, 28);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gateOn), ci.useGate);

	GtkObject *spinbutton_adj = gtk_adjustment_new(ci.percent, 0, 100, 1, 10, 10);
	GtkWidget *spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(spinbutton_adj), 1, 0);
	gtk_widget_ref(spinbutton);
	gtk_object_set_data_full(GTK_OBJECT(win), "spinbutton", spinbutton,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(spinbutton);
	gtk_fixed_put(GTK_FIXED(fixed), spinbutton, 368, 40);
	gtk_widget_set_uposition(spinbutton, 368, 40);
	gtk_widget_set_usize(spinbutton, 55, 26);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinbutton), TRUE);

	GtkWidget *pc = gtk_label_new(_("%"));
	gtk_widget_ref(pc);
	gtk_object_set_data_full(GTK_OBJECT(win), "pc", pc,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(pc);
	gtk_fixed_put(GTK_FIXED(fixed), pc, 424, 48);
	gtk_widget_set_uposition(pc, 424, 48);
	gtk_widget_set_usize(pc, 45, 20);

	GtkWidget *tunerLabel = gtk_label_new(_("Tuner"));
	gtk_widget_ref(tunerLabel);
	gtk_object_set_data_full(GTK_OBJECT(win), "tunerLabel", tunerLabel,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(tunerLabel);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(nb),
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), 0), tunerLabel);
	gtk_misc_set_padding(GTK_MISC(tunerLabel), 5, 0);
}

void
Interface::addAudioConf(GtkWidget *win, GtkWidget *nb) throw()
{
	Tuner::AudioInfo ai = GuiTuner::getConf()->getAudioInfo();
	Tuner::AudioDriver ad = GuiTuner::getConf()->getAudioDriver();
	Tuner::AudioDriver adp = GuiTuner::getConf()->getAudioDriverPlay();

	GtkWidget *fixed = gtk_fixed_new();
	gtk_widget_ref(fixed);
	gtk_object_set_data_full(GTK_OBJECT(win), "fixed", fixed,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fixed);
	gtk_container_add(GTK_CONTAINER(nb), fixed);

#ifndef HAVE_LIBASOUND

	GtkWidget *srs = gtk_option_menu_new();
	gtk_widget_ref(srs);
	gtk_object_set_data_full(GTK_OBJECT(win), "srs", srs,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(srs);
	gtk_fixed_put(GTK_FIXED(fixed), srs, 152, 16);
	gtk_widget_set_uposition(srs, 152, 16);
	gtk_widget_set_usize(srs, 100, 30);
	GtkWidget *srs_menu = gtk_menu_new();

	int val = 0;
	for(int i = 0; i < 5; ++i) {
		srsOpt[i].wid = gtk_menu_item_new_with_label(_(srsOpt[i].content));
		gtk_widget_show(srsOpt[i].wid);
		gtk_menu_append(GTK_MENU(srs_menu), srsOpt[i].wid);

		if(ai.rate == atoi(srsOpt[i].content))
			val = i;
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(srs), srs_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(srs), val);

	GtkWidget *sr = gtk_label_new(_("Sampling Rate:"));
	gtk_widget_ref(sr);
	gtk_object_set_data_full(GTK_OBJECT(win), "sr", sr,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(sr);
	gtk_fixed_put(GTK_FIXED(fixed), sr, 8, 16);
	gtk_widget_set_uposition(sr, 8, 16);
	gtk_widget_set_usize(sr, 140, 30);

	GtkWidget *fmts = gtk_option_menu_new();
	gtk_widget_ref(fmts);
	gtk_object_set_data_full(GTK_OBJECT(win), "fmts", fmts,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fmts);
	gtk_fixed_put(GTK_FIXED(fixed), fmts, 152, 56);
	gtk_widget_set_uposition(fmts, 152, 56);
	gtk_widget_set_usize(fmts, 100, 30);
	GtkWidget *fmts_menu = gtk_menu_new();

	for(int i = 0; i < 2; ++i) {
		fmtOpt[i].wid = gtk_menu_item_new_with_label(_(fmtOpt[i].content));
		gtk_widget_show(fmtOpt[i].wid);
		gtk_menu_append(GTK_MENU(fmts_menu), fmtOpt[i].wid);
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(fmts), fmts_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(fmts), ai.fmt == Tuner::U8 ? 0: 1);

	GtkWidget *fmt = gtk_label_new(_("Format:"));
	gtk_widget_ref(fmt);
	gtk_object_set_data_full(GTK_OBJECT(win), "fmt", fmt,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fmt);
	gtk_fixed_put(GTK_FIXED(fixed), fmt, 40, 56);
	gtk_widget_set_uposition(fmt, 40, 56);
	gtk_widget_set_usize(fmt, 80, 30);

	GtkWidget *hz = gtk_label_new(_("Hz"));
	gtk_widget_ref(hz);
	gtk_object_set_data_full(GTK_OBJECT(win), "hz", hz,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hz);
	gtk_fixed_put(GTK_FIXED(fixed), hz, 256, 16);
	gtk_widget_set_uposition(hz, 256, 16);
	gtk_widget_set_usize(hz, 42, 30);

#else
	GSList *audrGroup = 0;

	GtkWidget *ossBtn = gtk_radio_button_new_with_label(audrGroup, _("OSS"));
	audrGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(ossBtn));
	gtk_widget_ref(ossBtn);
	gtk_object_set_data_full(GTK_OBJECT(win), "ossBtn", ossBtn,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ossBtn);
	gtk_fixed_put(GTK_FIXED(fixed), ossBtn, 24, 40);
	gtk_widget_set_uposition(ossBtn, 24, 40);
	gtk_widget_set_usize(ossBtn, 115, 22);
	if(ad == Tuner::OSS_DRIVER)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ossBtn), TRUE);

	GtkWidget *alsaBtn = gtk_radio_button_new_with_label(audrGroup, _("Alsa"));
	audrGroup = gtk_radio_button_group(GTK_RADIO_BUTTON(alsaBtn));
	gtk_widget_ref(alsaBtn);
	gtk_object_set_data_full(GTK_OBJECT(win), "alsaBtn", alsaBtn,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(alsaBtn);
	gtk_fixed_put(GTK_FIXED(fixed), alsaBtn, 24, 72);
	gtk_widget_set_uposition(alsaBtn, 24, 72);
	gtk_widget_set_usize(alsaBtn, 115, 22);
	if(ad == Tuner::ALSA_DRIVER)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alsaBtn), TRUE);


	GtkWidget *audrLab = gtk_label_new(_("Audio Driver:"));
	gtk_widget_ref(audrLab);
	gtk_object_set_data_full(GTK_OBJECT(win), "audrLab", audrLab,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(audrLab);
	gtk_fixed_put(GTK_FIXED(fixed), audrLab, 8, 8);
	gtk_widget_set_uposition(audrLab, 8, 8);
	gtk_widget_set_usize(audrLab, 100, 22);

	GtkWidget *vsep = gtk_vseparator_new();
	gtk_widget_ref(vsep);
	gtk_object_set_data_full(GTK_OBJECT(win), "vsep", vsep,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vsep);
	gtk_fixed_put(GTK_FIXED(fixed), vsep, 232, 0);
	gtk_widget_set_uposition(vsep, 232, 0);
	gtk_widget_set_usize(vsep, 11, 164);

	GtkWidget *srs = gtk_option_menu_new();
	gtk_widget_ref(srs);
	gtk_object_set_data_full(GTK_OBJECT(win), "srs", srs,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(srs);
	gtk_fixed_put(GTK_FIXED(fixed), srs, 360, 16);
	gtk_widget_set_uposition(srs, 360, 16);
	gtk_widget_set_usize(srs, 86, 31);
	GtkWidget *srs_menu = gtk_menu_new();

	int val = 0;
	for(int i = 0; i < 5; ++i) {
		srsOpt[i].wid = gtk_menu_item_new_with_label(_(srsOpt[i].content));
		gtk_widget_show(srsOpt[i].wid);
		gtk_menu_append(GTK_MENU(srs_menu), srsOpt[i].wid);

		if(ai.rate == atoi(srsOpt[i].content))
			val = i;
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(srs), srs_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(srs), val);

	GtkWidget *sr = gtk_label_new(_("Sampling Rate:"));
	gtk_widget_ref(sr);
	gtk_object_set_data_full(GTK_OBJECT(win), "sr", sr,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(sr);
	gtk_fixed_put(GTK_FIXED(fixed), sr, 240, 24);
	gtk_widget_set_uposition(sr, 240, 24);
	gtk_widget_set_usize(sr, 116, 20);

	GtkWidget *fmts = gtk_option_menu_new();
	gtk_widget_ref(fmts);
	gtk_object_set_data_full(GTK_OBJECT(win), "fmts", fmts,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fmts);
	gtk_fixed_put(GTK_FIXED(fixed), fmts, 360, 56);
	gtk_widget_set_uposition(fmts, 360, 56);
	gtk_widget_set_usize(fmts, 86, 31);
	GtkWidget *fmts_menu = gtk_menu_new();

	for(int i = 0; i < 2; ++i) {
		fmtOpt[i].wid = gtk_menu_item_new_with_label(_(fmtOpt[i].content));
		gtk_widget_show(fmtOpt[i].wid);
		gtk_menu_append(GTK_MENU(fmts_menu), fmtOpt[i].wid);
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(fmts), fmts_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(fmts), ai.fmt == Tuner::U8 ? 0: 1);

	GtkWidget *fmt = gtk_label_new(_("Format:"));
	gtk_widget_ref(fmt);
	gtk_object_set_data_full(GTK_OBJECT(win), "fmt", fmt,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(fmt);
	gtk_fixed_put(GTK_FIXED(fixed), fmt, 240, 64);
	gtk_widget_set_uposition(fmt, 240, 64);
	gtk_widget_set_usize(fmt, 120, 20);

	GtkWidget *hz = gtk_label_new(_("Hz"));
	gtk_widget_ref(hz);
	gtk_object_set_data_full(GTK_OBJECT(win), "hz", hz,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hz);
	gtk_fixed_put(GTK_FIXED(fixed), hz, 448, 24);
	gtk_widget_set_uposition(hz, 448, 24);
	gtk_widget_set_usize(hz, 33, 20);

#endif

	GtkWidget *useEsd = gtk_check_button_new_with_label(_("Use ESD for output"));
	gtk_widget_ref(useEsd);
	gtk_object_set_data_full(GTK_OBJECT(win), "useEsd", useEsd,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(useEsd);
	gtk_fixed_put(GTK_FIXED(fixed), useEsd, 8, 120);
	gtk_widget_set_uposition(useEsd, 8, 120);
	gtk_widget_set_usize(useEsd, 170, 26);
	if(adp == Tuner::ESD_DRIVER)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(useEsd), TRUE);

	GtkWidget *audioLabel = gtk_label_new(_("Audio"));
	gtk_widget_ref(audioLabel);
	gtk_object_set_data_full(GTK_OBJECT(win), "audioLabel", audioLabel,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(audioLabel);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(nb),
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), 1), audioLabel);
	gtk_misc_set_padding(GTK_MISC(audioLabel), 5, 0);
}

void
Interface::addOtherConf(GtkWidget *win, GtkWidget *nb) throw()
{
	Tuner::NoteInfo ni = GuiTuner::getConf()->getNoteInfo();

	GtkWidget *table = gtk_table_new(2, 4, FALSE);
	gtk_widget_ref(table);
	gtk_object_set_data_full(GTK_OBJECT(win), "table", table,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(table);
	gtk_container_add(GTK_CONTAINER(nb), table);

	GtkWidget *freqLabel = gtk_label_new(_("\"A\" Frequency:"));
	gtk_widget_ref(freqLabel);
	gtk_object_set_data_full(GTK_OBJECT(win), "freqLabel", freqLabel,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(freqLabel);
	gtk_table_attach(GTK_TABLE(table), freqLabel, 0, 1, 0, 1,
		(GtkAttachOptions)(0),
		(GtkAttachOptions)(0), 0, 0);
	gtk_misc_set_padding(GTK_MISC(freqLabel), 10, 0);

	GtkWidget *hscale = gtk_hscale_new(GTK_ADJUSTMENT
		(gtk_adjustment_new(ni.ref, 220, 880, 0.1, 1, 0)));
	gtk_widget_ref(hscale);
	gtk_object_set_data_full(GTK_OBJECT(win), "hscale", hscale,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hscale);
	gtk_table_attach(GTK_TABLE(table), hscale, 1, 2, 0, 1,
		(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions)(GTK_FILL), 0, 0);
	gtk_widget_set_usize(hscale, 200, -2);
	gtk_scale_set_digits(GTK_SCALE(hscale), 2);

	GtkWidget *hz = gtk_label_new(_("Hz"));
	gtk_widget_ref(hz);
	gtk_object_set_data_full(GTK_OBJECT(win), "hz", hz,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hz);
	gtk_table_attach(GTK_TABLE(table), hz, 2, 3, 0, 1,
		(GtkAttachOptions)(0),
		(GtkAttachOptions)(0), 0, 0);
	gtk_misc_set_padding(GTK_MISC(hz), 5, 0);

	GtkWidget *defFreq = gtk_button_new_with_label(_("Default"));
	gtk_widget_ref(defFreq);
	gtk_object_set_data_full(GTK_OBJECT(win), "defFreq", defFreq,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(defFreq);
	gtk_table_attach(GTK_TABLE(table), defFreq, 3, 4, 0, 1,
		(GtkAttachOptions)(0),
		(GtkAttachOptions)(0), 0, 0);
	gtk_widget_set_usize(defFreq, 75, 30);
	gtk_container_set_border_width(GTK_CONTAINER(defFreq), 2);

	GtkWidget *nc = gtk_label_new(_("Naming Convention:"));
	gtk_widget_ref(nc);
	gtk_object_set_data_full(GTK_OBJECT(win), "nc", nc,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(nc);
	gtk_table_attach(GTK_TABLE(table), nc, 0, 1, 1, 2,
		(GtkAttachOptions)(0),
		(GtkAttachOptions)(0), 0, 0);
	gtk_misc_set_padding(GTK_MISC(nc), 5, 0);

	GtkWidget *ncDef = gtk_button_new_with_label(_("Default"));
	gtk_widget_ref(ncDef);
	gtk_object_set_data_full(GTK_OBJECT(win), "ncDef", ncDef,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ncDef);
	gtk_table_attach(GTK_TABLE(table), ncDef, 3, 4, 1, 2,
		(GtkAttachOptions)(0),
		(GtkAttachOptions)(0), 0, 0);
	gtk_widget_set_usize(ncDef, 75, 30);
	gtk_container_set_border_width(GTK_CONTAINER(ncDef), 2);

	GtkWidget *ncs = gtk_option_menu_new();
	gtk_widget_ref(ncs);
	gtk_object_set_data_full(GTK_OBJECT(win), "ncs", ncs,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ncs);
	gtk_table_attach(GTK_TABLE(table), ncs, 1, 2, 1, 2,
		(GtkAttachOptions)(GTK_FILL),
		(GtkAttachOptions)(0), 0, 0);
	gtk_container_set_border_width(GTK_CONTAINER(ncs), 2);
	GtkWidget *ncs_menu = gtk_menu_new();

	for(int i = 0; i < 2; ++i) {
		ncOpt[i].wid = gtk_menu_item_new_with_label(_(ncOpt[i].content));
		gtk_widget_show(ncOpt[i].wid);
		gtk_menu_append(GTK_MENU(ncs_menu), ncOpt[i].wid);
	}

	gtk_option_menu_set_menu(GTK_OPTION_MENU(ncs), ncs_menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(ncs), ni.conv == Tuner::ENGLISH ? 0: 1);

	GtkWidget *otherLabel = gtk_label_new(_("Other"));
	gtk_widget_ref(otherLabel);
	gtk_object_set_data_full(GTK_OBJECT(win), "otherLabel", otherLabel,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(otherLabel);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(nb),
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), 2), otherLabel);
	gtk_misc_set_padding(GTK_MISC(otherLabel), 5, 0);

	gtk_signal_connect(GTK_OBJECT(defFreq), "clicked",
		GTK_SIGNAL_FUNC(onDefClicked), 0);
	gtk_signal_connect(GTK_OBJECT(ncDef), "clicked",
		GTK_SIGNAL_FUNC(onDefClicked), (gpointer)1);
}

int
Interface::onDefClicked(GtkWidget *w, gpointer d)
{
	int i = (int)d;
	GtkWidget *wid;

	switch(i) {
		case 0:
			wid = lookupWidget(w, "hscale");
			gtk_adjustment_set_value(GTK_SCALE(wid)->range.adjustment, 440.00);
			break;
		case 1:
			wid = lookupWidget(w, "ncs");
			gtk_option_menu_set_history(GTK_OPTION_MENU(wid), 0);
			break;
		default:;
	}

	return TRUE;
}

int
Interface::onPrefClicked(GtkWidget *w, gpointer p)
{
	Tuner::DetectionMode dm;
	Tuner::AudioDriver ad, adp;

	Tuner::CalcInfo ci;
	Tuner::AudioInfo ai;
	Tuner::NoteInfo ni;

	ci.loadDefaults();
	ai.loadDefaults();
	ni.loadDefaults();

	collectTunerInfo(w, dm, ci);
	collectAudioInfo(w, ad, adp, ai);
	collectOtherInfo(w, ni);

	gtk_widget_destroy(static_cast<GtkWidget *>(p));

	GuiTuner::reconfigure(ad, adp, dm, &ai, &ai, &ci, &ni);

	return TRUE;
}

int
Interface::onPrefCancClicked(GtkWidget *wid, gpointer p)
{
	gtk_widget_destroy(static_cast<GtkWidget *>(p));

	return TRUE;
}

void
Interface::collectTunerInfo(GtkWidget *w, Tuner::DetectionMode &dm, Tuner::CalcInfo &ci)
{
	dm = Tuner::FFTCALC;

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
		(lookupWidget(w, "harmButt"))))
		ci.mode = Tuner::PEAK_SEARCH;
	else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
		(lookupWidget(w, "maxButt"))))
		ci.mode = Tuner::MAX_SEARCH;

	GtkWidget *wid = gtk_menu_get_active(GTK_MENU
		(GTK_OPTION_MENU(lookupWidget(w, "ptsMenu"))->menu));
	for(int i = 0; i < 3; ++i)
		if(ptsOpt[i].wid == wid)
			ci.points = atoi(ptsOpt[i].content);

	wid = gtk_menu_get_active(GTK_MENU
		(GTK_OPTION_MENU(lookupWidget(w, "winMenu"))->menu));
	for(int i = 0; i < 7; ++i)
		if(winOpt[i].wid == wid)
			ci.wtype = Tuner::Window::WindowType(i);

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
		(lookupWidget(w, "gateOn"))))
		ci.useGate = true;
	else
		ci.useGate = false;

	ci.percent = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
		(lookupWidget(w, "spinbutton")));
}

void
Interface::collectAudioInfo(GtkWidget * w, Tuner::AudioDriver &ad,
		Tuner::AudioDriver &adp, Tuner::AudioInfo &ai)
{
	GtkWidget *wid = gtk_menu_get_active(GTK_MENU
		(GTK_OPTION_MENU(lookupWidget(w, "srs"))->menu));
	for(int i = 0; i < 5; ++i)
		if(srsOpt[i].wid == wid)
			ai.rate = atoi(srsOpt[i].content);

	wid = gtk_menu_get_active(GTK_MENU
		(GTK_OPTION_MENU(lookupWidget(w, "fmts"))->menu));
	for(int i = 0; i < 2; ++i)
		if(fmtOpt[i].wid == wid)
			ai.fmt = i == 0 ? Tuner::U8: 
#if __BYTE_ORDER == __BIG_ENDIAN
			Tuner::S16BE;
#endif
#if __BYTE_ORDER == __LITTLE_ENDIAN
			Tuner::S16LE;
#endif

#ifdef HAVE_LIBASOUND

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
		(lookupWidget(w, "alsaBtn"))))
		ad = Tuner::ALSA_DRIVER;
	else
		ad = Tuner::OSS_DRIVER;

#else
	ad = adp = Tuner::OSS_DRIVER;
#endif
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
		(lookupWidget(w, "useEsd"))))
		adp = Tuner::ESD_DRIVER;
	else
		adp = ad;
}

void
Interface::collectOtherInfo(GtkWidget * w, Tuner::NoteInfo &ni)
{
	ni.ref = GTK_HSCALE(lookupWidget(w, "hscale"))->scale.range.adjustment->value;
	GtkWidget *wid = gtk_menu_get_active(GTK_MENU
		(GTK_OPTION_MENU(lookupWidget(w, "ncs"))->menu));
	for(int i = 0; i < 2; ++i)
		if(ncOpt[i].wid == wid)
			ni.conv = Tuner::NamingConv(i);
}

