/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOMEBASE_H_
#define _GNOMEBASE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <gnome.h>

namespace Config {
	class GenConf;
}

namespace Interface {
	class GnomeBase {
		public:
		enum DialogType { INFO, ERROR };

			GnomeBase(int &, char **&) throw();
			~GnomeBase();

		bool start() throw();
		bool stop() throw();

		static void openMessageBox(const std::string &, DialogType) throw();
		static void openPlayWindow(Config::GenConf &) throw();
		static void openPrefWindow() throw();

		protected:
		bool isLooping() const throw();

		private:
		bool gtk_looping;
	};

	inline GnomeBase::GnomeBase(int &ac, char **&av) throw(): gtk_looping(false)
	{
		gnome_init("GuiTuner", VERSION, ac, av);
	}
	inline GnomeBase::~GnomeBase()
	{
		if(gtk_looping)
			gtk_main_quit();
	}
	inline bool GnomeBase::start() throw()
	{
		if(gtk_looping)
			return false;
		gtk_looping = true;
		gtk_main();
		return true;
	}
	inline bool GnomeBase::stop() throw()
	{
		if(!gtk_looping)
			return false;
		gtk_main_quit();
		gtk_looping = false;
		return true;
	}
	inline bool GnomeBase::isLooping() const throw() { return gtk_looping; }

}

#endif

