/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h> 
#endif

#include <iostream>
#include <stdexcept>
#include <string>
#include <unistd.h>
#include "error.h"
#include "gnomegui.h"
#include "textgui.h"
#include "guituner.h"

const int DefaultRefresh = 300;
GuiTuner *GuiTuner::obj = 0;
bool GuiTuner::running = false;

GuiTuner::GuiTuner(int &argc, char **&argv):
	gui(0), mt(0), conf(0)
{
	using namespace Interface;

	try {
#ifdef ENABLE_NLS
		bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
		textdomain(PACKAGE);
#endif

		gui = new GnomeGui(&argc, &argv);
	} catch(...) {
		gui = new TextGui(&argc, &argv);
	}

	try {
		conf = new Config::GenConf(argc, argv);
		gui->open();
		mt = new Thread::MainThread(gui, *conf, DefaultRefresh);
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot start application ")) + e.what());
		exit(-1);
	}

	obj = this;
}

GuiTuner::~GuiTuner()
{
	if(running)
		stop();

	delete mt;
	delete conf;
	delete gui;

	obj = 0;
}

void
GuiTuner::start()
{
	try {
		running = true;
		obj->mt->start();
		obj->gui->start();
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot start threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::stop()
{
	try {
		obj->mt->stop();
		obj->gui->close();
		running = false;
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot stop threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::pause()
{
	try {
		obj->mt->pause();
		obj->gui->pause();
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot pause threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::resume()
{
	try {
		obj->gui->resume();
		obj->mt->resume();
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot restart application ")) + e.what());
		exit(-1);
	}
}

void 
GuiTuner::reconfigure(Tuner::AudioDriver ad, Tuner::AudioDriver adp,
	Tuner::DetectionMode dm, Tuner::AudioInfo *ai, Tuner::AudioInfo *aip,
	Tuner::CalcInfo *ci, Tuner::NoteInfo *ni)
{
	try {
		obj->conf->setInfo(ad, adp, dm, ai, aip, ci, ni);
		obj->mt->reconfigure(DefaultRefresh);
	} catch(std::exception &e) {
		reportMsg(std::string(_("Cannot configure application "))
			 + e.what());
		exit(-1);
	}
}

Config::GenConf *
GuiTuner::getConf() throw()
{
	return obj->conf;
}

void
GuiTuner::reportMsg(const std::string &msg) throw()
{
	std::cerr << msg << endl;
}

