/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _OBJFACTORY_H_
#define _OBJFACTORY_H_

#include "audioerror.h"
#include "calcerror.h"
#include "genconf.h"

namespace Tuner {
	class Audio;
	class Calc;
	template <typename T, typename S> class Buffer;

	class ObjFactory {
		public:
			ObjFactory(Config::GenConf &) throw();
			~ObjFactory();

		AudioInfo *getAudioInfo() throw();
		bool setup(const AudioInfo *) throw(std::bad_alloc);
		bool update() throw(Error::AudioError, Error::CalcError, std::bad_alloc);

	 	Audio *&createAudio() throw(Error::AudioError, std::bad_alloc);
		Audio *&createAudioPlay() throw(Error::AudioError, std::bad_alloc);
		Calc *&createCalc(const AudioInfo * = 0) throw(Error::CalcError, std::bad_alloc);
		Buffer<unsigned char, unsigned int> *&createBuffer(
			const AudioInfo * = 0)throw(std::bad_alloc);

		private:
			ObjFactory(const ObjFactory &);
		const ObjFactory &operator=(const ObjFactory &);

		Config::GenConf &config;
		Audio *audio, *audioPlay;
		Calc *calc;
		Buffer<unsigned char, unsigned int> *buffer;
	};

	inline AudioInfo *ObjFactory::getAudioInfo() throw() { return &config.getAudioInfo(); }
}

#endif

