/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include "conf.h"
#include "utils.h"

void
Tuner::char2int(const unsigned char *ibuf, int *obuf, int len, ByteOrder order) throw()
{
	int i;
	len /= 2;
	switch(order) {
		case LITTLEENDIAN:
			for(i = 0; i < len; ++i)
				obuf[i] = (ibuf[i * 2] |
				(static_cast<int>(ibuf[i * 2 + 1]) << 8)) - 32768;
			break;
		case BIGENDIAN:
			for(i = 0; i < len; ++i)
				obuf[i] = (ibuf[i * 2 + 1] |
				(static_cast<int>(ibuf[i * 2]) << 8)) - 32768;
	}
}

bool
Tuner::tone(Buffer<unsigned char> &buf, double freq, const AudioInfo &ai) throw()
{
	const int Seconds = 3;
	const int CharAmp = 120;
	const int IntAmp = 31096;

	unsigned char *ptr;
	double period = ai.rate / freq;

	if(ai.chn != MONO)
		return false;
	if(ai.fmt == U8) {
		int outLen = Seconds * ai.rate;
		buf.resize(outLen);
		ptr = buf.g_ptr();
		for(int i = 0; i < outLen; ++i)
			*ptr++ = static_cast<unsigned char>(128 + CharAmp *
				sin((2 * M_PI * static_cast<double>(i)) / period));
	} else {
		int outLen = Seconds * ai.rate * 2;
		buf.resize(outLen);
		ptr = buf.g_ptr();
		switch(ai.fmt) {
		case S16LE:
			for(int i = 0; i < outLen / 2; ++i) {
				int val = static_cast<int>(IntAmp *
					sin((2 * M_PI * static_cast<double>(i)) / period));
				*ptr++ = static_cast<unsigned int>(val) & 0x00FF;
				*ptr++ = (static_cast<unsigned int>(val) & 0xFF00) >> 8;
			}
			break;
			case S16BE:
			for(int i = 0; i < outLen / 2; ++i) {
				int val = static_cast<int>(IntAmp *
					sin((2 * M_PI * static_cast<double>(i)) / period));
				*ptr++ = (static_cast<unsigned int>(val) & 0xFF00) >> 8;
				*ptr++ = static_cast<unsigned int>(val) & 0x00FF;
			}
			break;
			default:
				return false;
		}
	}

	return true;
}

